@@ -14,6 +14,7 @@
 #    under the License.
 
 from django.conf import settings
+from django.core.urlresolvers import reverse
 from django.forms import ValidationError  # noqa
 from django.utils.translation import ugettext_lazy as _
 from oslo_utils import strutils
@@ -37,6 +38,151 @@ ST_EXTRA_SPECS_FORM_ATTRS = {
 }
 
 
+class MigrationStart(forms.SelfHandlingForm):
+    name = forms.CharField(
+        label=_("Share Name"), required=False,
+        widget=forms.TextInput(attrs={'readonly': 'readonly'}))
+    share_id = forms.CharField(
+        label=_("ID"), required=False,
+        widget=forms.TextInput(attrs={'readonly': 'readonly'}))
+    host = forms.CharField(
+        max_length=255, label=_("Host to migrate share"),
+        help_text=_("Destination host where share will be migrated to. Use the"
+                    " format 'host@backend#pool'."))
+    force_host_assisted_migration = forms.BooleanField(
+        label=_("Force Host Assisted Migration"),
+        required=False, initial=False,
+        help_text=("Defines whether the migration of this share should skip "
+                   "the attempt to be assisted by a storage vendor backend. "
+                   "This will force the use of the Data Service to perform "
+                   "migration."))
+    nondisruptive = forms.BooleanField(
+        label=_("Non-disruptive"),
+        required=False, initial=False,
+        help_text=("Defines whether the migration of this share should be "
+                   "performed only if it is non-disruptive If set so, this "
+                   "will prevent the use of the Data Service for migration."))
+    writable = forms.BooleanField(
+        label=_("Writable"), required=False, initial=True,
+        help_text=("Defines whether this share should remain writable during "
+                   "migration. If set so, this will prevent the use of the "
+                   "Data Service for migration."))
+    preserve_metadata = forms.BooleanField(
+        label=_("Preserve Metadata"), required=False, initial=True,
+        help_text=("Defines whether this share should have all its file "
+                   "metadata preserved during migration. If set so, this will "
+                   "prevent the use of the Data Service for migration."))
+    new_share_network = forms.ChoiceField(
+        label=_("New share network to be set in migrated share"),
+        required=False,
+        help_text=_("Input the new share network where the share should"
+                    " be migrated to if you would like to change it."))
+    new_share_type = forms.ChoiceField(
+        label=_("New share type to be set in migrating share"), required=False,
+        help_text=_("Input the new share type which the migrating share will "
+                    "be set to if you would like to change the type."))
+
+    def __init__(self, request, *args, **kwargs):
+        super(MigrationStart, self).__init__(request, *args, **kwargs)
+        share_networks = manila.share_network_list(request)
+        share_types = manila.share_type_list(request)
+        st_choices = [('', '')] + [(st.id, st.name) for st in share_types]
+        sn_choices = (
+            [('', '')] +
+            [(sn.id, sn.name or sn.id) for sn in share_networks])
+        self.fields['new_share_type'].choices = st_choices
+        self.fields['new_share_network'].choices = sn_choices
+
+    def handle(self, request, data):
+        share_name = _get_id_if_name_empty(data)
+        try:
+            manila.migration_start(
+                request, self.initial['share_id'],
+                force_host_assisted_migration=(
+                    data['force_host_assisted_migration']),
+                writable=data['writable'],
+                preserve_metadata=data['preserve_metadata'],
+                nondisruptive=data['nondisruptive'],
+                dest_host=data['host'],
+                new_share_network_id=data['new_share_network'],
+                new_share_type_id=data['new_share_type'])
+
+            messages.success(
+                request,
+                _('Successfully sent the request to migrate share: %s.')
+                % share_name)
+            return True
+        except Exception:
+            redirect = reverse("horizon:admin:shares:index")
+            exceptions.handle(
+                request, _("Unable to migrate share %s.") % share_name,
+                redirect=redirect)
+        return False
+
+
+class MigrationForms(forms.SelfHandlingForm):
+    name = forms.CharField(
+        label=_("Share Name"), required=False,
+        widget=forms.TextInput(attrs={'readonly': 'readonly'}))
+    share_id = forms.CharField(
+        label=_("ID"), required=False,
+        widget=forms.TextInput(attrs={'readonly': 'readonly'}))
+
+
+class MigrationComplete(MigrationForms):
+
+    def handle(self, request, data):
+        share_name = _get_id_if_name_empty(data)
+        try:
+            manila.migration_complete(request, self.initial['share_id'])
+            messages.success(
+                request,
+                _('Successfully sent the request to complete migration of '
+                  ' share: %s.') % share_name)
+            return True
+        except Exception:
+            exceptions.handle(request, _("Unable to complete migration "
+                                         "of share %s.") % share_name)
+        return False
+
+
+class MigrationGetProgress(MigrationForms):
+
+    def handle(self, request, data):
+        share_name = _get_id_if_name_empty(data)
+        try:
+            result = manila.migration_get_progress(request,
+                                                   self.initial['share_id'])
+            progress = result[1]
+            messages.success(
+                request,
+                _('Migration of share %(name)s is at %(progress)s percent.') %
+                {'name': share_name, 'progress': progress['total_progress']})
+            return True
+        except Exception:
+            exceptions.handle(request, _("Unable to obtain progress of "
+                                         "migration of share %s at this "
+                                         "moment.") % share_name)
+        return False
+
+
+class MigrationCancel(MigrationForms):
+
+    def handle(self, request, data):
+        share_name = _get_id_if_name_empty(data)
+        try:
+            manila.migration_cancel(request, self.initial['share_id'])
+            messages.success(
+                request,
+                _('Successfully sent the request to cancel migration of '
+                  ' share: %s.') % share_name)
+            return True
+        except Exception:
+            exceptions.handle(request, _("Unable to cancel migration of share"
+                                         " %s at this moment.") % share_name)
+        return False
+
+
 class ManageShare(forms.SelfHandlingForm):
     name = forms.CharField(
         max_length=255, label=_("Share Name"), required=False,
@@ -54,10 +200,9 @@ class ManageShare(forms.SelfHandlingForm):
         max_length=255, label=_("Export location"), required=True,
         help_text=_("Export location of share. Example for NFS: "
                     "1.2.3.4:/path/to/share"))
-    protocol = forms.ChoiceField(
-        label=_("Share Protocol"), required=True,
-        choices=(('NFS', 'NFS'), ('CIFS', 'CIFS'), ('GlusterFS', 'GlusterFS'),
-                 ('HDFS', 'HDFS'), ('CephFS', 'CephFS')))
+
+    protocol = forms.ChoiceField(label=_("Share Protocol"), required=True)
+
     share_type = forms.ChoiceField(label=_("Share Type"), required=True)
 
     driver_options = forms.CharField(
@@ -82,6 +227,17 @@ class ManageShare(forms.SelfHandlingForm):
             if dhss and dhss.lower() in strutils.FALSE_STRINGS:
                 st_choices.append((st.name, st.name))
         self.fields['share_type'].choices = st_choices
+        # NOTE(vkmc): choose only those share protocols that are enabled
+        # FIXME(vkmc): this should be better implemented by having a
+        # capabilities endpoint on the control plane.
+        manila_features = getattr(settings, 'OPENSTACK_MANILA_FEATURES', {})
+        self.enabled_share_protocols = manila_features.get(
+            'enabled_share_protocols',
+            ['NFS', 'CIFS', 'GlusterFS', 'HDFS', 'CephFS'])
+        self.fields['protocol'].choices = ([(' ', ' ')] +
+                                           [(enabled_proto, enabled_proto)
+                                           for enabled_proto in
+                                           self.enabled_share_protocols])
 
     def handle(self, request, data):
         try:
@@ -323,3 +479,12 @@ class CreateShareNetworkForm(forms.SelfHandlingForm):
             exceptions.handle(request,
                               _('Unable to create share network.'))
             return False
+
+
+def _get_id_if_name_empty(data):
+    result = data.get('name', None)
+    if not result:
+        result = data.get('id')
+    if not result:
+        result = ''
+    return result
