@@ -13,10 +13,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-"""
-Views for managing shares.
-"""
-
+from django.conf import settings
 from django.core.urlresolvers import reverse
 from django.forms import ValidationError  # noqa
 from django.template.defaultfilters import filesizeformat  # noqa
@@ -30,7 +27,6 @@ from horizon.utils.memoized import memoized  # noqa
 
 from manila_ui.api import manila
 from manila_ui.dashboards import utils
-from openstack_dashboard.api import nova
 from openstack_dashboard.usage import quotas
 
 
@@ -51,15 +47,23 @@ class CreateForm(forms.SelfHandlingForm):
 
     def __init__(self, request, *args, **kwargs):
         super(CreateForm, self).__init__(request, *args, **kwargs)
-        share_protos = ('NFS', 'CIFS', 'GlusterFS', 'HDFS', 'CephFS')
+        # NOTE(vkmc): choose only those share protocols that are enabled
+        # FIXME(vkmc): this should be better implemented by having a
+        # capabilities endpoint on the control plane.
+        manila_features = getattr(settings, 'OPENSTACK_MANILA_FEATURES', {})
+        self.enabled_share_protocols = manila_features.get(
+            'enabled_share_protocols',
+            ['NFS', 'CIFS', 'GlusterFS', 'HDFS', 'CephFS'])
+        self.enable_public_shares = manila_features.get(
+            'enable_public_shares', True)
         share_networks = manila.share_network_list(request)
         share_types = manila.share_type_list(request)
         self.fields['share_type'].choices = (
             [("", "")] + [(st.name, st.name) for st in share_types])
-        availability_zones = nova.availability_zone_list(request)
+
+        availability_zones = manila.availability_zone_list(request)
         self.fields['availability_zone'].choices = (
-            [("", "")] +
-            [(az.zoneName, az.zoneName) for az in availability_zones])
+            [("", "")] + [(az.name, az.name) for az in availability_zones])
 
         self.sn_field_name_prefix = 'share-network-choices-'
         for st in share_types:
@@ -99,12 +103,15 @@ class CreateForm(forms.SelfHandlingForm):
         self.fields['metadata'] = forms.CharField(
             label=_("Metadata"), required=False,
             widget=forms.Textarea(attrs={'rows': 4}))
-        self.fields['is_public'] = forms.BooleanField(
-            label=_("Make visible for all"), required=False,
-            help_text=(
-                "If set then all tenants will be able to see this share."))
 
-        self.fields['share_proto'].choices = [(sp, sp) for sp in share_protos]
+        if self.enable_public_shares:
+            self.fields['is_public'] = forms.BooleanField(
+                label=_("Make visible for all"), required=False,
+                help_text=(
+                    "If set then all tenants will be able to see this share."))
+
+        self.fields['share_proto'].choices = [(sp, sp) for sp in
+                                              self.enabled_share_protocols]
         if "snapshot_id" in request.GET:
             try:
                 snapshot = self.get_snapshot(request,
@@ -220,6 +227,7 @@ class CreateForm(forms.SelfHandlingForm):
             except ValidationError as e:
                 self.api_error(e.messages[0])
                 return False
+            is_public = self.enable_public_shares and data['is_public']
             share = manila.share_create(
                 request,
                 size=data['size'],
@@ -229,7 +237,7 @@ class CreateForm(forms.SelfHandlingForm):
                 share_network=share_network,
                 snapshot_id=snapshot_id,
                 share_type=data['share_type'],
-                is_public=data['is_public'],
+                is_public=is_public,
                 metadata=metadata,
                 availability_zone=data['availability_zone'])
             message = _('Creating share "%s"') % data['name']
@@ -259,13 +267,22 @@ class UpdateForm(forms.SelfHandlingForm):
         widget=forms.Select(
             attrs={'class': 'switched', 'data-slug': 'sharetype'}))
 
+    def __init__(self, request, *args, **kwargs):
+        super(UpdateForm, self).__init__(request, *args, **kwargs)
+        manila_features = getattr(settings, 'OPENSTACK_MANILA_FEATURES', {})
+        self.enable_public_shares = manila_features.get(
+            'enable_public_shares', True)
+        if not self.enable_public_shares:
+            self.fields.pop('is_public')
+
     def handle(self, request, data):
         share_id = self.initial['share_id']
+        is_public = data['is_public'] if self.enable_public_shares else False
         try:
             share = manila.share_get(self.request, share_id)
             manila.share_update(
                 request, share, data['name'], data['description'],
-                is_public=data['is_public'])
+                is_public=is_public)
             message = _('Updating share "%s"') % data['name']
             messages.success(request, message)
             return True
